local classes = require "classes"
local defines = require "defines"
local Button = classes.class()

function Button:init(buttontype, texture, texturehighlighted, x, y, xsize, ysize, mouse, collider)
	self.type = buttontype
	self.x = x
	self.y = y

	self.dx = 0

	self.drawable = love.graphics.newImage(texture)
	self.drawablehighlighted = love.graphics.newImage(texturehighlighted)
	self.collision = collider:addRectangle(x, y, xsize, ysize)
	self.collision.type = TYPE_PLAYER
	self.collision.index = buttontype

	self.mouse = mouse

	self.enabled = false
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Get

function Button:isHighlighted()
	return (self.enabled and self.collision:collidesWith(self.mouse))
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Set

function Button:changeDirection(direction)
	if (direction == MENU_LEFT) then
		self.dx = -MENU_MOVESPEED
	elseif (direction == MENU_RIGHT) then
		self.dx = MENU_MOVESPEED
	else
		self.dx = 0
	end
end

function Button:enableButton()
	self.enabled = true
end

function Button:disableButton()
	self.enabled = false
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Other

function Button:move(dt)
	if (self.dx ~= 0) then
		self.x = self.x + (self.dx * dt)
	end
end

function Button:draw()
	--g_debugtext = tostring(self:isHighlighted())
	--self.collision:draw('fill')

	if (self:isHighlighted()) then
		love.graphics.draw(self.drawablehighlighted, self.x, self.y)
	else
		love.graphics.draw(self.drawable, self.x, self.y)
	end
end

return Button

