local classes = require "classes"
local defines = require "defines"
local StraightEnemy = require "StraightEnemy"
local StraightBullet = require "StraightBullet"
local Power = require "Power"
local Player = require "Player"
local Sound = require "Sound"

local Command = classes.class()

function Command:init()
	-- Command
	self.commandlist = {}
	self.music = Sound.new()

	self.numcommands = 0
	self.currentcommand = 0
	self.wait = -1
	self.ending = false
	self.repeating = false
	self.laststart = 0
	self.lastend = 0

	-- Object/Bullet Management
	self.enemylist = {}
	self.bulletlist = {}
	self.powerlist = {}
	self.playerbulletlist = {}

	self.currentbullet = 0
	self.currentpower = 0
	self.currentplayerbullet = 0

	-- Player
	self.player = nil
	self.slowspeed = 100
	self.normalspeed = 300
	self.fastspeed = 500

	-- Power
	self.stayuplength = 0
	self.stayalivelength = 0
	self.xsize = 0
	self.ysize = 0
	self.initialspeed = 0
	self.dy = 0
	self.texture = ""
	self.texturecollected = ""
	self.poweramount = 0
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Command Management

function Command:addCommand(commandtotal)
	-- Remove whitespace (ie tabbing) from the string
	commandtotal = commandtotal:match'^%s*(.*)'
	local command = split(commandtotal, " ")

	-- Ignore comments (though unless it starts with a keyword it's ignored anyways....)
	if ((command[1] == "//") or (command[0] == "")) then
		return
	end

	self.commandlist[self.numcommands] = {}
	if (command[1] == "CREATE") then
		self.commandlist[self.numcommands].command = COMMAND_CREATE
		if (command[2] == "STRAIGHTENEMY") then
			self.commandlist[self.numcommands].type = TYPE_ENEMY
			self.commandlist[self.numcommands].index = tonumber(command[3])
			self.commandlist[self.numcommands].x = tonumber(command[4])
			self.commandlist[self.numcommands].y = tonumber(command[5])
			self.commandlist[self.numcommands].xsize = tonumber(command[6])
			self.commandlist[self.numcommands].ysize = tonumber(command[7])
			self.commandlist[self.numcommands].speed = tonumber(command[8])
			self.commandlist[self.numcommands].texture = command[9]
			self.commandlist[self.numcommands].hp = tonumber(command[10])
			if (command[10] == "true") then
				self.commandlist[self.numcommands].droppower = true
			else
				self.commandlist[self.numcommands].droppower = false
			end
		end
	elseif (command[1] == "SHOOT") then
		if (command[2] == "STRAIGHT") then
			self.commandlist[self.numcommands].command = COMMAND_SHOOT
			self.commandlist[self.numcommands].type = TYPE_STRAIGHTBULLET
			self.commandlist[self.numcommands].index = tonumber(command[3])
			self.commandlist[self.numcommands].x = tonumber(command[4])
			self.commandlist[self.numcommands].y = tonumber(command[5])
			self.commandlist[self.numcommands].xsize = tonumber(command[6])
			self.commandlist[self.numcommands].ysize = tonumber(command[7])
			self.commandlist[self.numcommands].speed = tonumber(command[8])
			self.commandlist[self.numcommands].texture = command[9]
			self.commandlist[self.numcommands].size = tonumber(command[10])
			self.commandlist[self.numcommands].offset = tonumber(command[11])
		elseif (command[2] == "PLAYER") then
			self.commandlist[self.numcommands].command = COMMAND_SHOOT
			self.commandlist[self.numcommands].type = TYPE_STRAIGHTBULLETPLAYER
			self.commandlist[self.numcommands].index = tonumber(command[3])
			self.commandlist[self.numcommands].xsize = tonumber(command[4])
			self.commandlist[self.numcommands].ysize = tonumber(command[5])
			self.commandlist[self.numcommands].speed = tonumber(command[6])
			self.commandlist[self.numcommands].texture = command[7]
			self.commandlist[self.numcommands].size = tonumber(command[8])
			self.commandlist[self.numcommands].offset = tonumber(command[9])
		elseif (command[2] == "SLANTED") then
			self.commandlist[self.numcommands].command = COMMAND_SHOOT
			self.commandlist[self.numcommands].type = TYPE_STRAIGHTBULLET
			self.commandlist[self.numcommands].index = tonumber(command[3])
			self.commandlist[self.numcommands].x = tonumber(command[4])
			self.commandlist[self.numcommands].y = tonumber(command[5])
			self.commandlist[self.numcommands].xsize = tonumber(command[6])
			self.commandlist[self.numcommands].ysize = tonumber(command[7])
			self.commandlist[self.numcommands].speed = tonumber(command[8])
			self.commandlist[self.numcommands].texture = command[9]
			self.commandlist[self.numcommands].slantx = tonumber(command[10])
			self.commandlist[self.numcommands].slanty = tonumber(command[11])
			self.commandlist[self.numcommands].size = tonumber(command[12])
			self.commandlist[self.numcommands].offset = tonumber(command[13])
		end
	elseif (command[1] == "MOVE") then
		self.commandlist[self.numcommands].command = COMMAND_MOVE
		self.commandlist[self.numcommands].index = tonumber(command[2])
		self.commandlist[self.numcommands].x = tonumber(command[3])
		self.commandlist[self.numcommands].y = tonumber(command[4])
		self.commandlist[self.numcommands].speed = tonumber(command[5])
	elseif (command[1] == "WAIT") then
		self.commandlist[self.numcommands].command = COMMAND_WAIT
		self.commandlist[self.numcommands].waittime = tonumber(command[2])
	elseif (command[1] == "REPEAT") then
		self.commandlist[self.numcommands].command = COMMAND_REPEAT
		self.laststart = self.numcommands
		self.commandlist[self.numcommands].index = tonumber(command[2])
		self.commandlist[self.numcommands].repeatstart = self.numcommands
	elseif (command[1] == "STOP") then
		self.commandlist[self.numcommands].command = COMMAND_STOP
		self.commandlist[self.numcommands].repeatstart = self.laststart
		self.commandlist[self.numcommands].repeatend = self.numcommands
		self.commandlist[self.laststart].repeatend = self.numcommands
	elseif (command[1] == "KILL") then
		self.commandlist[self.numcommands].command = COMMAND_KILL
		self.commandlist[self.numcommands].index = tonumber(command[2])
	elseif (command[1] == "KILLALL") then
		self.commandlist[self.numcommands].command = COMMAND_KILLALL
	elseif (command[1] == "SWAY") then
		self.commandlist[self.numcommands].command = COMMAND_SWAY
		self.commandlist[self.numcommands].index = tonumber(command[2])
		self.commandlist[self.numcommands].degree = tonumber(command[3])
		self.commandlist[self.numcommands].speed = tonumber(command[4])
	elseif (command[1] == "MUSIC") then
		self.commandlist[self.numcommands].command = COMMAND_MUSIC
		self.commandlist[self.numcommands].path = command[2]
	elseif (command[1] == "END") then
		self.commandlist[self.numcommands].command = COMMAND_END
	end

	self.numcommands = self.numcommands + 1
end

function Command:executeCommands(dt)
	--g_debugtext = tostring(self.laststart).." "..tostring(self.lastend)

	-- Make sure the game isn't over
	if (self.ending) then
		return
	end

	-- If we're currently in wait mode, then do so
	if (self.commandlist[self.currentcommand].command == COMMAND_WAIT) then
		if (self.wait > 0) then
			self.wait = self.wait - dt
			return
		else
			self.currentcommand = self.currentcommand + 1
		end
	end

	-- Make sure we don't go over our limit
	if (self.currentcommand >= self.numcommands) then
		return
	end

	-- Cache
	local command = self.commandlist[self.currentcommand]

	-- If we're repeating, if the enemy is dead then just skip to the end
	if (self.repeating) then
		if (self.enemylist[self.commandlist[self.laststart].index] == nil) then
			self.currentcommand = self.lastend
			self.repeating = false
			return
		end
	end

	-- Parse commands
	while (command.command ~= COMMAND_WAIT) do
		if (command.command == COMMAND_CREATE) then
			self.enemylist[command.index] = StraightEnemy.new(command.x, command.y, command.xsize, command.ysize, command.speed, command.texture, command.index, command.hp)
			if (command.droppower) then
				-- Removed it from the constructor because it would be too long... Just ugly.. Others are already very long (>80 chars)
				self.enemylist[command.index]:setDropPower(self.stayuplength, self.stayalivelength, self.xsize, self.ysize, self.initialspeed, self.dy, self.texture, self.poweramount)
			end
		elseif (command.command == COMMAND_SHOOT) then
			if (command.type == TYPE_STRAIGHTBULLET) then
				if (self:checkAndUpdateEnemyBullets()) then
					if (self.enemylist[command.index] ~= nil) then
						if (not self.enemylist[command.index]:isDead()) then
							local angle = math.atan2((command.y - self.enemylist[command.index]:getY()), (command.x - self.enemylist[command.index]:getX())) + (command.offset * (math.pi / 180))
							self.bulletlist[self.currentbullet] = StraightBullet.new(self.enemylist[command.index]:getX() + (self.enemylist[command.index]:getXSize() / 2), self.enemylist[command.index]:getY() + (self.enemylist[command.index]:getYSize() / 2), math.cos(angle), math.sin(angle), command.xsize, command.ysize, command.texture, self.currentbullet, TYPE_ENEMYBULLET, command.speed, 0, 0, command.size, 0)
							self.currentbullet = self.currentbullet + 1
						end
					end
				end
			elseif (command.type == TYPE_STRAIGHTBULLETPLAYER) then
				if (self:checkAndUpdateEnemyBullets()) then
					if (self.enemylist[command.index] ~= nil) then
						if (not self.enemylist[command.index]:isDead()) then
							local angle = math.atan2((self.player:getY() - self.enemylist[command.index]:getY()), (self.player:getX() - self.enemylist[command.index]:getX())) + (command.offset * (math.pi / 180))
							self.bulletlist[self.currentbullet] = StraightBullet.new(self.enemylist[command.index]:getX() + (self.enemylist[command.index]:getXSize() / 2), self.enemylist[command.index]:getY() + (self.enemylist[command.index]:getYSize() / 2), math.cos(angle), math.sin(angle), command.xsize, command.ysize, command.texture, self.currentbullet, TYPE_ENEMYBULLET, command.speed, 0, 0, command.size, 0)
							self.currentbullet = self.currentbullet + 1
						end
					end
				end
			elseif (command.type == TYPE_SLANTEDBULLET) then
				if (self:checkAndUpdateEnemyBullets()) then
					if (self.enemylist[command.index] ~= nil) then
						if (not self.enemylist[command.index]:isDead()) then
							local angle = math.atan2((getPlayerID():getY() - self.enemylist[command.index]:getY()), (getPlayerID():getX() - self.enemylist[command.index]:getX())) + (command.offset * (math.pi / 180))
							self.bulletlist[self.currentbullet] = StraightBullet.new(self.enemylist[command.index]:getX(), self.enemylist[command.index]:getY(), math.cos(angle), math.sin(angle), command.xsize, command.ysize, command.texture, self.currentbullet, TYPE_ENEMYBULLET, command.speed, command.slantx, command.slanty, command.size, 0)
							self.currentbullet = self.currentbullet + 1
						end
					end
				end
			end
		elseif (command.command == COMMAND_MOVE) then
			if (self.enemylist[command.index] ~= nil) then
				self.enemylist[command.index]:setMoveTo(command.x, command.y, command.speed)
			end
		elseif (command.command == COMMAND_REPEAT) then
			self.laststart = command.repeatstart
			self.lastend = command.repeatend
			self.repeating = true
		elseif (command.command == COMMAND_STOP) then
			if (self.enemylist[self.commandlist[self.laststart].index] ~= nil) then
				self.currentcommand = command.repeatstart
			else
				self.repeating = false
			end
		elseif (command.command == COMMAND_KILL) then
			if (self.enemylist[command.index] ~= nil) then
				self:dealDamage(command.index, 999999)
			end
		elseif (command.command == COMMAND_KILLALL) then
			for i = 0, MAX_ENEMIES, 1 do
				self:dealDamage(i, 999999)
			end
		elseif (command.command == COMMAND_SWAY) then
			if (self.enemylist[command.index] ~= nil) then
				self.enemylist[command.index]:setSway(command.degree, command.speed)
			end
		elseif (command.command == COMMAND_MUSIC) then
			self.music:playSound(command.path)
		elseif (command.command == COMMAND_END) then
			self.ending = true
			return
		end

		self.currentcommand = self.currentcommand + 1
		command = self.commandlist[self.currentcommand]
	end

	-- Go into waiting mode
	if (command.command == COMMAND_WAIT) then
		self.wait = command.waittime
	end
end

function Command:deleteAllCommands(command)
	self.commandlist = {}
	self.numcommands = 0
	self.currentcommand = 0
	self.wait = -1
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Object management

function Command:update(dt)
	-- Objects
	for i = 0, MAX_ENEMYBULLETS, 1 do
		if (self.bulletlist[i] ~= nil) then
			self.bulletlist[i]:move(dt)
		end
	end

	for i = 0, MAX_ENEMIES, 1 do
		if (self.enemylist[i] ~= nil) then
			self.enemylist[i]:move(dt)
		end
	end

	if (g_state == STATE_GAME) then
		for i = 0, MAX_POWER, 1 do
			if (self.powerlist[i] ~= nil) then
				self.powerlist[i]:move(dt, self.player)
			end
		end

		for i = 0, MAX_BULLETS, 1 do
			if (self.playerbulletlist[i] ~= nil) then
				self.playerbulletlist[i]:move(dt)
			end
		end

		-- Player
		self.player:UpdateFireTimer(dt)
		self.player:updateMouse(love.mouse.getX(), love.mouse.getY())
		self.player:move(dt)
	end
end

function Command:draw()
	for i = 0, MAX_ENEMIES, 1 do
		if (self.enemylist[i] ~= nil) then
			self.enemylist[i]:draw()
		end
	end

	for i = 0, MAX_ENEMYBULLETS, 1 do
		if (self.bulletlist[i] ~= nil) then
			self.bulletlist[i]:draw()
		end
	end

	for i = 0, MAX_POWER, 1 do
		if (self.powerlist[i] ~= nil) then
			self.powerlist[i]:draw()
		end
	end

	for i = 0, MAX_BULLETS, 1 do
		if (self.playerbulletlist[i] ~= nil) then
			self.playerbulletlist[i]:draw()
		end
	end

	if (g_state == STATE_GAME) then
		self.player:draw()
	end
end

function Command:createPower(x, y)
	if (self:checkAndUpdatePower()) then
		if (self.powerlist[self.currentpower] == nil) then
			self.powerlist[self.currentpower] = Power.new(x, y, self.xsize, self.ysize, self.stayuplength, self.stayalivelength, self.initialspeed, self.dy, self.texture, self.texturecollected, self.currentpower, self.poweramount)
			self.currentbullet = self.currentbullet + 1
		end
	end
end

function Command:setPowerCollected(index)
	if (self.powerlist[index] ~= nil) then
		self.powerlist[index]:setCollected()
	end
end

function Command:deletePower(index)
	Collider:remove(self.powerlist[index].collision)
	self.powerlist[index] = nil
end

function Command:setDefaultPower(stayuplength, stayalivelength, xsize, ysize, initialspeed, dy, texture, texturecollected, poweramount)
	self.stayuplength = stayuplength
	self.stayalivelength = stayalivelength
	self.xsize = xsize
	self.ysize = ysize
	self.initialspeed = initialspeed
	self.dy = dy
	self.texture = texture
	self.texturecollected = texturecollected
	self.poweramount = poweramount
end

function Command:dealDamage(index)
	if (self.enemylist[index] ~= nil) then
		self.enemylist[index]:takeDamage(self.player:getDamage())

		if (self.enemylist[index]:isDead()) then
			self.enemylist[index]:setDying()
		end
	end
end

function Command:deleteBullet(index)
	Collider:remove(self.bulletlist[index].collision)
	self.bulletlist[index] = nil
end

function Command:deletePlayerBullet(index)
	Collider:remove(self.playerbulletlist[index].collision)
	self.playerbulletlist[index] = nil
end

function Command:deleteEnemy(index)
	Collider:remove(self.enemylist[index].collision)
	self.enemylist[index] = nil
end

function Command:checkAndUpdateEnemyBullets()
	local counter = 0
	while true do
		-- Infinite loop = bad
		if (counter > MAX_ENEMYBULLETS) then
			return false
		end

		if (self.currentbullet > MAX_ENEMYBULLETS) then
			self.currentbullet = 0
		elseif (self.bulletlist[self.currentbullet] == nil) then
			return true
		else
			self.currentbullet = self.currentbullet + 1
			counter = counter + 1
		end
	end
end

function Command:checkAndUpdatePower()
	local counter = 0
	while true do
		if (counter > MAX_POWER) then
			return false
		end

		if (self.powerlist[self.currentpower] == nil) then
			return true
		elseif (self.currentpower > MAX_POWER) then
			self.currentpower = 0
		else
			self.currentpower = self.currentpower + 1
			counter = counter + 1
		end
	end
end

-- Return true if there's an available bullet slot, false otherwise
function Command:checkAndUpdatePlayerBullets()
	local counter = 0
	while true do
		if (counter > MAX_BULLETS) then
			return false
		end

		if (self.playerbulletlist[self.currentplayerbullet] == nil) then
			return true
		elseif (self.currentplayerbullet > MAX_BULLETS) then
			self.currentplayerbullet = 0
		else
			self.currentplayerbullet = self.currentplayerbullet + 1
			counter = counter + 1
		end
	end
end

function Command:stopMusic()
	self.music:stopSound()
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Player Management

function Command:firePlayerBullet()
	if (self:checkAndUpdatePlayerBullets()) then
		self.playerbulletlist[self.currentplayerbullet] = self.player:Fire(self.currentplayerbullet)
	end
end

function Command:createPlayer(startx, starty, xsize, ysize, speed, texture, damage, bullettexture, slowspeed, fastspeed)
	self.slowspeed = slowspeed
	self.normalspeed = speed
	self.fastspeed = fastspeed

	self.player = Player.new(startx, starty, xsize, ysize, speed, texture, damage, bullettexture)
end

function Command:updateMouse(x, y)
	self.player:updateMouse(x, y)
end

function Command:setPlayerSpeed(speed)
	if (speed == SPEED_SLOW) then
		self.player:setSpeed(self.slowspeed)
	elseif (speed == SPEED_NORMAL) then
		self.player:setSpeed(self.normalspeed)
	else
		self.player:setSpeed(self.fastspeed)
	end
end

function Command:fixPositionWall(walltype)
	self.player:fixPositionWall(walltype)
end

function Command:playerGetHit(dt)
	self.player:getHit(dt)
end

function Command:checkFireTimer()
	return self.player:checkFireTimer()
end

function Command:increaseDamage(damage)
	self.player:addDamage(damage)
end

-----------------------------------------------------------------------------------------
-- http://lua-users.org/wiki/SplitJoin
-- Apparently it still has infinite loop glitches, but we shouldn't get that here as long as the files don't derp...
function split(str, pat)
   local t = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pat
   local last_end = 1
   local s, e, cap = str:find(fpat, 1)
   while s do
      if s ~= 1 or cap ~= "" then
	 table.insert(t,cap)
      end
      last_end = e+1
      s, e, cap = str:find(fpat, last_end)
   end
   if last_end <= #str then
      cap = str:sub(last_end)
      table.insert(t, cap)
   end
   return t
end

return Command