local classes = require "classes"
local defines = require "defines"
local Button = require "Button"
local Command = require "Command"
local Menu = classes.class()

function Menu:init()
	self.command = Command.new()

	self.menustate = MENUSTATE_MAIN
	self.menustatemoveto = MENUSTATE_MAIN
	self.decorationlist = {}
	self.mainlist = {}
	self.optionslist = {}

	self.mousex = love.mouse.getX()
	self.mousey = love.mouse.getY()

	self.collision = nil

	self.fading = true
	self.fadingin = true
	self.currentfade = 255
	self.fadingamount = MENU_FADETIME

	love.graphics.setBackgroundColor(0, 175, 175)

	self.transition = false
	self.transitiontime = MENU_TRANSITIONTIME

	self.teardown = false
end

function Menu:addCommand(command)
	self.command:addCommand(command)
end

function Menu:addCollision()
	self.collision = Collider:addRectangle(self.mousex, self.mousey, 2, 2)
	self.collision.type = TYPE_MOUSE
end

function Menu:addButton(buttontype, texture, texturehighlighted, x, y, xsize, ysize)
	if ((buttontype == BUTTON_QUIT) or (buttontype == BUTTON_OPTIONS) or (buttontype == BUTTON_PLAY)) then
		self.mainlist[buttontype] = Button.new(buttontype, texture, texturehighlighted, x, y, xsize, ysize, self.collision, Collider)
	elseif (buttontype == BUTTON_DECORATION) then
		self.decorationlist[buttontype] = Button.new(buttontype, texture, texturehighlighted, x, y, xsize, ysize, self.collision, Collider)
	end
end

function Menu:update(dt)
	self.mousex = love.mouse.getX()
	self.mousey = love.mouse.getY()

	if (self.fading) then
		if (self.fadingin) then
			--self.music:fadeIn(dt, self.fadingamount)

			self.currentfade = self.currentfade - ((255 / self.fadingamount) * dt)

			if (self.currentfade < 1) then
				self.currentfade = 0
				self.fading = false
				self.fadingin = false
			end
		else
			self.currentfade = self.currentfade + ((255 / self.fadingamount) * dt)
			if (self.currentfade > 254) then
				self.currentfade = 255
				self.fading = false
				self.fadingin = true

				love.graphics.setBackgroundColor(0, 0, 0)
				self:tearDownMenu()
				beginGame()
			end
		end
	end

	if (self.transition) then
		if (self.transitiontime <= 0) then
			self:moveButtons(MENU_STOP, self.menustate)
			if (self.menustatemoveto ~= MENUSTATE_PLAY) then
				self:moveButtons(MENU_STOP, self.menustatemoveto)
				self.transition = false
				self.transitiontime = MENU_TRANSITIONTIME
			end
		else
			local currentlist = self:getCurrentList(self.menustate)

			for i = 0, MAX_MENUS, 1 do
				if (currentlist[i] ~= nil) then
					currentlist[i]:move(dt)
				end
			end

			currentlist = self:getCurrentList(self.menustatemoveto)

			for i = 0, MAX_MENUS, 1 do
				if (currentlist[i] ~= nil) then
					currentlist[i]:move(dt)
				end
			end
		end
	end

	if (self.command ~= nil) then
		self.command:executeCommands(dt)
		self.command:update(dt)
	end

	self.collision:moveTo(self.mousex, self.mousey)
end

function Menu:draw()
	self.command:draw()

	if (not self.teardown) then
		if (self.fading) then
			love.graphics.setColor(0, 0, 0, self.currentfade)
			love.graphics.rectangle("fill", 0, 0, 1024, 768)
		end

		love.graphics.setColor(255, 255, 255, 255-self.currentfade)

		local currentlist = self:getCurrentList(self.menustate)

		for i = 0, MAX_MENUS, 1 do
			if (currentlist[i] ~= nil) then
				currentlist[i]:draw()
			end
		end

		for i = 0, MAX_MENUS, 1 do
			if (self.decorationlist[i] ~= nil) then
				self.decorationlist[i]:draw()
			end
		end
	end
end

function Menu:tearDownMenu()
	self.command:stopMusic()
	self.teardown = true
	self.command = nil
end

function Menu:handleClick()
	local currentlist = self:getCurrentList(self.menustate)

	if (not self.intransition) then
		for i = 0, MAX_MENUS, 1 do
			if (currentlist[i] ~= nil) then
				if (currentlist[i]:isHighlighted()) then -- If the mouse is over this button
					if (currentlist[i].type == BUTTON_QUIT) then
						love.event.push('quit')
					elseif (currentlist[i].type == BUTTON_OPTIONS) then
						self:swapTo(BUTTON_OPTIONS)
					elseif (currentlist[i].type == BUTTON_PLAY) then
						self:swapTo(BUTTON_PLAY)
					end
				end
			end
		end
	end
end

function Menu:swapTo(buttontype)
	self:disableAllButtons()

	self.transition = true

	if (buttontype == BUTTON_OPTIONS) then
		self.menustatemoveto = MENUSTATE_OPTIONS
		self:moveButtons(MENU_LEFT, self.menustate)
		self:moveButtons(MENU_RIGHT, self.menustatemoveto)
	elseif (buttontype == BUTTON_PLAY) then
		self.fadingin = false
		self.fading = true
		self.menustatemoveto = MENUSTATE_PLAY
		self:moveButtons(MENU_LEFT, self.menustate)
	end
end

function Menu:moveButtons(direction, statetype)
	local currentlist = self:getCurrentList(statetype)

	for i = 0, MAX_MENUS, 1 do
		if (currentlist[i] ~= nil) then
			currentlist[i]:changeDirection(direction)
		end
	end
end

function Menu:enableType(statetype)
	local currentlist = self:getCurrentList(statetype)

	for i = 0, MAX_MENUS, 1 do
		if (currentlist[i] ~= nil) then
			currentlist[i]:enableButton()
		end
	end
end

function Menu:disableAllButtons()
	for i = 0, MAX_MENUS, 1 do
		if (self.mainlist[i] ~= nil) then
			self.mainlist[i]:disableButton()
		end

		if (self.optionslist[i] ~= nil) then
			self.optionslist[i]:disableButton()
		end
	end
end

function Menu:deleteBullet(index)
	self.command:deleteBullet(index)
end

function Menu:onCollide(dt, shape_a, shape_b)
	-- local shape1 = self:collisionIsMouse(shape_a)
	-- local shape2 = self:collisionIsMouse(shape_b)

	-- if (shape1 or shape2) then
	-- 	local mouse
	-- 	local other

	-- 	if (shape1) then
	-- 		mouse = shape1
	-- 		other = shape2
	-- 	else
	-- 		mouse = shape2
	-- 		other = shape1
	-- 	end


	-- end
end

function Menu:getCurrentList(statetype)
	if (statetype == MENUSTATE_MAIN) then
		return self.mainlist
	elseif (statetype == MENUSTATE_OPTIONS) then
		return self.optionslist
	elseif (statetype == MENUSTATE_PLAY) then
		return {}
	end	
end

function Menu:collisionIsMouse(shape)
	return (shape.type == TYPE_MOUSE)
end

return Menu