local classes = require "classes"
local defines = require 'defines'
local StraightBullet = require 'StraightBullet'

local Player = classes.class()

function Player:init(x, y, xsize, ysize, speed, texture, damage, bullettexture)
	self.x = x
	self.y = y
	self.xsize = xsize
	self.ysize = ysize
	self.speed = speed
	self.drawable = love.graphics.newImage(texture)
	self.collision = Collider:addRectangle(x-(xsize/4), y-(ysize/4), (xsize/2), (ysize/2))
	self.collision.type = TYPE_PLAYER
	self.collision.index = 0

	self.bullettexture = bullettexture

	self.mousex = 0
	self.mousey = 0
	self.firetimer = 0.1
	self.firetimerreset = 0.1
	self.canfire = true

	self.lastmovex = 0
	self.lastmovey = 0
	self.lastmovedt = 0

	self.lives = 3

	self.damage = damage

	self.invulnerable = false
	self.invulnerabletimer = TIME_INVULNERABLE

	self.isdead = false
	self.invisible = false

	self.speedtype = SPEED_NORMAL
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Get

function Player:getLivesLeft()
	return self.lives
end

function Player:getX()
	return self.x
end

function Player:getY()
	return self.y
end

function Player:getDamage()
	return self.damage
end

function Player:checkFireTimer(dx)
	return self.canfire
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Set

function Player:setLives(lives)
	self.lives = lives
end

function Player:addDamage(damage)
	self.damage = self.damage + damage

	if (self.damage > MAX_PLAYERDAMAGE) then
		self.damage = MAX_PLAYERD
	end
end

function Player:resetDamage(damage)
	self.damage = 1
end

function Player:setSpeed(speed)
	self.speed = speed
end

function Player:updateMouse(mousex, mousey)
	self.mousex = mousex
	self.mousey = mousey
end

function Player:fixPositionWall(wall)
	local movex = 0
	local movey = 0

	if ((wall == WALL_PLAYERTOP) or (wall == WALL_PLAYERBOTTOM)) then
		self.y = self.y - self.lastmovey
		movey = movey - self.lastmovey
	elseif ((wall == WALL_PLAYERLEFT) or (wall == WALL_PLAYERRIGHT)) then
		self.x = self.x - self.lastmovex
		movex = movex - self.lastmovex
	end

	self.collision:move(movex, movey)
end

function Player:Fire(bullet)
	self.canfire = false

	local angle = math.atan2((love.mouse.getY() - self.y), (love.mouse.getX() - self.x))
	return StraightBullet.new(self.x, self.y+5, math.cos(angle), math.sin(angle), 8, 8, self.bullettexture, bullet, TYPE_PLAYERBULLET, 800, 0, 0, self.damage * 0.8)
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Other

function Player:getHit(dt)
	if (not self.isdead) then
		self.isdead = true
		self.invulnerable = true
		self.invulnerabletimer = TIME_INVULNERABLE
		self.lives = self.lives - 1
		self.damage = 1
	end
end

function Player:move(dt)
	local movex = 0
	local movey = 0

	if (love.keyboard.isDown("w")) then
		movey = movey - (self.speed * dt)
		self.y = self.y - (self.speed * dt)
	elseif (love.keyboard.isDown("s")) then
		movey = movey + (self.speed * dt)
		self.y = self.y + (self.speed * dt)
	end

	if (love.keyboard.isDown("a")) then
		movex = movex - (self.speed * dt)
		self.x = self.x - (self.speed * dt)
	elseif (love.keyboard.isDown("d")) then
		movex = movex + (self.speed * dt)
		self.x = self.x + (self.speed * dt)
	end

	self.lastmovex = movex
	self.lastmovey = movey
	self.lastmovedt = dt
	self.collision:move(movex, movey)
end

function Player:UpdateFireTimer(dx)
	if (not self.canfire) then
		self.firetimer = self.firetimer - dx

		if (self.firetimer < 0) then
			self.firetimer = self.firetimerreset
			self.canfire = true
		end
	end
end

function Player:draw()
	if (self.invulnerable) then
		local dt = love.timer.getDelta()
		self.invulnerabletimer = self.invulnerabletimer - dt

		if (self.invulnerabletimer < 0) then
			self.invulnerable = false
			self.isdead = false
			self.invisible = false
		end
	end

	if (not self.isdead) then
		love.graphics.draw(self.drawable, self.x, self.y, math.atan2(self.y-self.mousey, self.x-self.mousex), 1, 1, self.xsize/2, self.ysize/2)
	else
		if (self.invisible) then
			love.graphics.draw(self.drawable, self.x, self.y, math.atan2(self.y-self.mousey, self.x-self.mousex), 1, 1, self.xsize/2, self.ysize/2)
			self.invisible = false
		else
			self.invisible = true
		end
	end

	if (self.speedtype == SPEED_SLOW) then
		self.collision:draw('fill')
	end
end

return Player