local classes = require "classes"
local defines = require 'defines'

local Power = classes.class()

function Power:init(x, y, xsize, ysize, stayuplength, stayalivelength, initialspeed, dy, texture, texturecollected, index, poweramount)
	self.x = x
	self.y = y
	self.xsize = xsize
	self.ysize = ysize
	self.staylength = stayuplength
	self.stayalivelength = stayalivelength
	self.speed = initialspeed					-- Should normally be 0 to make a speedup effect or a negative number to do a "popup"
	self.dx = 0
	self.dy = dy
	self.texture = texture
	self.index = index
	self.poweramount = poweramount
	self.type = TYPE_POWER

	self.drawable = love.graphics.newImage(texture)
	self.drawablecollected = love.graphics.newImage(texturecollected)
	self.collision = Collider:addRectangle(x - (self.xsize/3), y - (self.ysize/3), xsize, ysize)
	self.collision.type = TYPE_POWER
	self.collision.index = index

	self.movingdown = false
	self.collected = false
	self.deathtimer = 0.1
end

function Power:setCollected()
	self.collected = true
end

function Power:move(dt, player)
	if (self.collected) then
		self.deathtimer = self.deathtimer - dt

		if (self.deathtimer < 0) then
			increaseDamage(self.poweramount)
			removePower(self.collision.index)
			return
		end
		self.speed = 800

		local angle = math.atan2((player:getY() - self.y), (player:getX() - self.x))

		local movex = math.cos(angle) * dt * self.speed
		local movey = math.sin(angle) * dt * self.speed

		self.x = self.x + movex
		self.y = self.y + movey

		self.collision:move(movex, movey)
		return
	end

	if (not self.movingdown) then
		self.staylength = self.staylength - dt
		self.stayalivelength = self.stayalivelength - dt

		if (self.staylength < 0) then
			self.movingdown = true
		elseif (self.stayalivelength < 0) then
			removePower(self.index)
		end
	else
		self.poweramount = self.poweramount - (0.2 * dt)
		self.speed = self.speed + (self.dy * dt)

		self.y = self.y + self.speed
		self.collision:move(0, self.speed)
	end
end

function Power:draw()
	--self.collision:draw('fill')
	if (self.collected) then
		love.graphics.draw(self.drawablecollected, self.x, self.y, 0, self.poweramount * 3, self.poweramount * 3)
	else
		love.graphics.draw(self.drawable, self.x, self.y, 0, self.poweramount * 3, self.poweramount * 3)
	end
end

return Power