local classes = require "classes"
local defines = require 'defines'
local StraightEnemy = classes.class()

function StraightEnemy:init(x, y, xsize, ysize, speed, texture, index, hp)
	self.x = x
	self.y = y

	self.endx = 0
	self.endy = 0

	self.dx = 0
	self.dy = 0

	self.xsize = xsize
	self.ysize = ysize

	self.speed = speed

	self.drawable = love.graphics.newImage(texture)
	self.collision = Collider:addRectangle(x, y, xsize, ysize)
	self.collision.type = TYPE_ENEMY
	self.collision.index = index

	self.hp = hp

	self.moving = false
	self.movetime = -1

	self.dying = false
	self.shrinkamount = 1
	self.droppower = false

	self.swayamount = 0
	self.currentdirection = SWAY_COUNTERCLOCKWISE
	self.currentsway = 0
	self.swayspeed = 0

	--self.endx = endx
	--self.endy = endy
	--local angle = math.atan2((starty - endy), (startx - endx))
	--self.enddx = math.cos(angle)
	--self.enddy = math.sin(angle)
	--self.leavex = leavex
	--self.leavey = leavey
	--angle = math.atan2((endy - leavey), (endx - leavex))
	--self.leavedx = math.cos(angle)
	--self.leavedy = math.sin(angle)
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Get

function StraightEnemy:isDead()
	return (self.hp <= 0)
end

function StraightEnemy:getX()
	return self.x
end

function StraightEnemy:getY()
	return self.y
end

function StraightEnemy:getXSize()
	return self.xsize
end

function StraightEnemy:getYSize()
	return self.ysize
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Set

function StraightEnemy:setSway(sway, speed)
	self.swayamount = sway * (math.pi / 180) -- Convert to radians.. Easier to use
	self.swayspeed = speed * (math.pi / 180) -- Speed is degrees per second.. Convert to radians as well
end

function StraightEnemy:setDropPower()
	self.droppower = true
end

function StraightEnemy:setDying()
	self.dying = true
end

function StraightEnemy:setMoveTo(x, y, speed)
	self.endx = x
	self.endy = y

	local angle = math.atan2((y - self.y), (x - self.x))
	self.dx = math.cos(angle)
	self.dy = math.sin(angle)

	self.movetime = math.sqrt((math.pow((x - self.x), 2) + math.pow((y - self.y), 2))) / speed

	self.speed = speed
	self.moving = true
end

function StraightEnemy:takeDamage(damage)
	self.hp = self.hp - damage
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-- Other

function StraightEnemy:move(dt)
	if (self.moving) then
		if (self.movetime > 0) then
			local movex = 0
			local movey = 0

			self.movetime = self.movetime - dt

			self.x = self.x + (self.dx * dt * self.speed)
			movex = movex + (self.dx * dt * self.speed)

			self.y = self.y + (self.dy * dt * self.speed)
			movey = movey + (self.dy * dt * self.speed)

			self.collision:move(movex, movey)
		else
			self.moving = false
		end
	end
end

function StraightEnemy:draw()
	--self.collision:draw('fill')
	love.graphics.draw(self.drawable, self.x, self.y, 0, self.shrinkamount, self.shrinkamount)

	if (self.dying) then
		self.shrinkamount = self.shrinkamount - 0.1

		if (self.shrinkamount < 0) then
			createPower(self.x, self.y)
			removeEnemy(self.collision.index)
		end
	end
end

return StraightEnemy